#include <stdio.h>

#include "gd.h"

gdImagePtr gd;

int rv;
int user_margin;
int user_width, user_height;
int no_trans;
int monochrome;

void
load(FILE *f)
{
  gd = gdImageCreateFromGif(f);
}

void
save(FILE *f)
{
  gdImageGif(gd, f);
  gdImageDestroy(gd);
}

void
process(char *fname)
{
  int w, h, x, y, sx, sy, p, tp=-1, cx=0, cy=0;
  gdImagePtr gd2;

  w = gdImageSX(gd);
  h = gdImageSY(gd);
  sx = 0;
  sy = 0;

  p = gdImageGetPixel(gd, 0, 0);

  for (y=h-1; y; y--)
  {
    for (x=0; x<w; x++)
      if (gdImageGetPixel(gd, x, y) != p)
	goto done_h;
    h = y;
    tp = p;
  }
 done_h:

  for (x=w-1; x; x--)
  {
    for (y=0; y<h; y++)
      if (gdImageGetPixel(gd, x, y) != p)
	goto done_w;
    w = x;
    tp = p;
  }
 done_w:

  for (x=0; x<w; x++)
  {
    for (y=0; y<h; y++)
      if (gdImageGetPixel(gd, x, y) != p)
	goto done_sx;
    sx = x+1;
    tp = p;
  }
 done_sx:

  for (y=0; y<h; y++)
  {
    for (x=0; x<w; x++)
      if (gdImageGetPixel(gd, x, y) != p)
	goto done_sy;
    sy = y+1;
    tp = p;
  }
 done_sy:


  if (user_margin != -1)
  {
    sx -= user_margin;
    sy -= user_margin;
    w += user_margin;
    h += user_margin;
  }

  if (user_width != -1)
  {
    int addl = (user_width - (w-sx))/2;
    int addr = user_width - (w-sx) - addl;
    sx -= addl;
    w += addr;
  }

  if (user_height != -1)
  {
    int addt = (user_height - (h-sy))/2;
    int addb = user_height - (h-sy) - addt;
    sy -= addt;
    h += addb;
  }

  fprintf(stderr, "%s [(%d,%d) - (%d,%d)] -> %d x %d\n",
	  fname, sx, sy, w, h, w-sx, h-sy);

  gd2 = gdImageCreate(w-sx, h-sy);

  gdImageInterlace(gd2, 1);
  if (tp == -1)
    tp = gdImageGetTransparent(gd);
  tp = gdImageColorAllocate(gd2,
			    gdImageRed(gd, tp),
			    gdImageGreen(gd, tp),
			    gdImageBlue(gd, tp));
  if (!no_trans)
    gdImageColorTransparent(gd2, tp);

  gdImageFilledRectangle(gd2, 0, 0, w-sx, h-sy, tp);

  if (sx < 0)
  {
    cx = -sx;
    sx = 0;
  }
  if (sy < 0)
  {
    cy = -sy;
    sy = 0;
  }
  if (w > gdImageSX(gd))
  {
    w = gdImageSX(gd);
  }
  if (h > gdImageSY(gd))
  {
    h = gdImageSY(gd);
  }

  gdImageCopy(gd2, gd, cx, cy, sx, sy, w-sx, h-sy);

  gdImageDestroy(gd);
  gd = gd2;

  if (monochrome)
  {
    int black = gdImageColorAllocate(gd, 0, 0, 0);
    int white = gdImageColorAllocate(gd, 255, 255, 255);
    int x, y, c;
    for (x=0; x<gdImageSX(gd); x++)
      for (y=0; y<gdImageSY(gd); y++)
      {
	c = gdImageGetPixel(gd, x, y);
	c = gdImageGreen(gd, c) * 12
	  + gdImageRed(gd, c) * 5
	  + gdImageBlue(gd, c) * 3;
	c /= 20;
	if (c < 85)
	  gdImageSetPixel(gd, x, y, black);
	else if (c > 171)
	  gdImageSetPixel(gd, x, y, white);
	else if ((x+y)&1)
	  gdImageSetPixel(gd, x, y, white);
	else
	  gdImageSetPixel(gd, x, y, black);
      }
  }
}

void
do_file(char *fname)
{
  FILE *f = fopen(fname, "rb");
  if (f == 0)
  {
    perror(fname);
    rv++;
    return;
  }
  load(f);
  fclose(f);

  process(fname);

  f = fopen(fname, "wb");
  if (f == 0)
  {
    perror(fname);
    rv++;
    return;
  }
  save(f);
  fclose(f);
}

int
main(int argc, char **argv)
{
  int i;

  user_margin = user_width = user_height = -1;
  no_trans = 0;
  rv = 0;

  while (argc > 1 && argv[1][0] == '-')
  {
    if (strcmp(argv[1], "-w") == 0)
      user_width = atoi(argv[2]);
    else if (strcmp(argv[1], "-h") == 0)
      user_height = atoi(argv[2]);
    else if (strcmp(argv[1], "-m") == 0)
      user_margin = atoi(argv[2]);
    else if (strcmp(argv[1], "-nt") == 0)
    {
      no_trans = 1;
      argc++;
      argv--;
    } else if (strcmp(argv[1], "-bw") == 0) {
      monochrome = 1;
    }
    argc -= 2;
    argv += 2;
  }

  if (argc > 1)
    for (i=1; i<argc; i++)
      do_file(argv[i]);
  else
  {
    load(stdin);
    process("<stdin>");
    save(stdout);
  }
  return rv;
}
